@@ -51,23 +51,23 @@ class MDirUpdate;
 class MDentryLink;
 class MDentryUnlink;
 class MLock;
-class MMDSFindIno;
-class MMDSFindInoReply;
-class MMDSOpenIno;
-class MMDSOpenInoReply;
+struct MMDSFindIno;
+struct MMDSFindInoReply;
+struct MMDSOpenIno;
+struct MMDSOpenInoReply;
 
 class Message;
 class MClientRequest;
 class MMDSSlaveRequest;
-class MClientSnap;
+struct MClientSnap;
 
 class MMDSFragmentNotify;
 
 class ESubtreeMap;
 
-class Mutation;
-class MDRequest;
-class MDSlaveUpdate;
+struct Mutation;
+struct MDRequest;
+struct MDSlaveUpdate;
 
 
 // flags for predirty_journal_parents()
@@ -801,8 +801,8 @@ protected:
   void handle_open_ino(MMDSOpenIno *m);
   void handle_open_ino_reply(MMDSOpenInoReply *m);
   friend class C_MDC_OpenInoBacktraceFetched;
-  friend class C_MDC_OpenInoTraverseDir;
-  friend class C_MDC_OpenInoParentOpened;
+  friend struct C_MDC_OpenInoTraverseDir;
+  friend struct C_MDC_OpenInoParentOpened;
 
 public:
   void kick_open_ino_peers(int who);
@@ -858,6 +858,7 @@ public:
 
   // -- stray --
 public:
+  void scan_stray_dir();
   void eval_stray(CDentry *dn);
   void eval_remote(CDentry *dn);
 
@@ -871,7 +872,6 @@ public:
       eval_stray(dn);
   }
 protected:
-  void scan_stray_dir(dirfrag_t next=dirfrag_t());
   void fetch_backtrace(inodeno_t ino, int64_t pool, bufferlist& bl, Context *fin);
   void remove_backtrace(inodeno_t ino, int64_t pool, Context *fin);
   void _purge_forwarding_pointers(bufferlist& bl, CDentry *dn, int r);
@@ -880,7 +880,6 @@ protected:
   void _purge_stray_purged(CDentry *dn, int r=0);
   void _purge_stray_logged(CDentry *dn, version_t pdv, LogSegment *ls);
   void _purge_stray_logged_truncate(CDentry *dn, LogSegment *ls);
-  friend class C_MDC_RetryScanStray;
   friend class C_MDC_FetchedBacktrace;
   friend class C_MDC_PurgeForwardingPointers;
   friend class C_MDC_PurgeStray;
