@@ -277,9 +277,9 @@ int SimpleMessenger::rebind(const set<int>& avoid_ports)
 {
   ldout(cct,1) << "rebind avoid " << avoid_ports << dendl;
   assert(did_bind);
-  accepter.stop();
+  int r = accepter.rebind(avoid_ports);
   mark_down_all();
-  return accepter.rebind(avoid_ports);
+  return r;
 }
 
 int SimpleMessenger::start()
