@@ -20,8 +20,33 @@ static ostream& _prefix(std::ostream* _dout)
   return *_dout << "--OSD::tracker-- ";
 }
 
-void OpHistory::insert(utime_t now, OpRequest *op)
+OpRequest::OpRequest(Message *req, OpTracker *tracker) :
+  request(req), xitem(this),
+  rmw_flags(0),
+  warn_interval_multiplier(1),
+  lock("OpRequest::lock"),
+  tracker(tracker),
+  hit_flag_points(0), latest_flag_point(0),
+  seq(0) {
+  received_time = request->get_recv_stamp();
+  tracker->register_inflight_op(&xitem);
+  if (req->get_priority() < g_conf->osd_client_op_priority) {
+    // don't warn as quickly for low priority ops
+    warn_interval_multiplier = g_conf->osd_recovery_op_warn_multiple;
+  }
+}
+
+void OpHistory::on_shutdown()
 {
+  arrived.clear();
+  duration.clear();
+  shutdown = true;
+}
+
+void OpHistory::insert(utime_t now, OpRequestRef op)
+{
+  if (shutdown)
+    return;
   duration.insert(make_pair(op->get_duration(), op));
   arrived.insert(make_pair(op->get_arrived(), op));
   cleanup(now);
@@ -30,11 +55,11 @@ void OpHistory::insert(utime_t now, OpRequest *op)
 void OpHistory::cleanup(utime_t now)
 {
   while (arrived.size() &&
-	 now - arrived.begin()->first > (double)(g_conf->osd_op_history_duration)) {
+	 (now - arrived.begin()->first >
+	  (double)(g_conf->osd_op_history_duration))) {
     duration.erase(make_pair(
 	arrived.begin()->second->get_duration(),
 	arrived.begin()->second));
-    delete arrived.begin()->second;
     arrived.erase(arrived.begin());
   }
 
@@ -42,7 +67,6 @@ void OpHistory::cleanup(utime_t now)
     arrived.erase(make_pair(
 	duration.begin()->second->get_arrived(),
 	duration.begin()->second));
-    delete duration.begin()->second;
     duration.erase(duration.begin());
   }
 }
@@ -55,7 +79,7 @@ void OpHistory::dump_ops(utime_t now, Formatter *f)
   f->dump_int("duration to keep", g_conf->osd_op_history_duration);
   {
     f->open_array_section("Ops");
-    for (set<pair<utime_t, const OpRequest *> >::const_iterator i =
+    for (set<pair<utime_t, OpRequestRef> >::const_iterator i =
 	   arrived.begin();
 	 i != arrived.end();
 	 ++i) {
@@ -68,31 +92,27 @@ void OpHistory::dump_ops(utime_t now, Formatter *f)
   f->close_section();
 }
 
-void OpTracker::dump_historic_ops(ostream &ss)
+void OpTracker::dump_historic_ops(Formatter *f)
 {
-  JSONFormatter jf(true);
   Mutex::Locker locker(ops_in_flight_lock);
   utime_t now = ceph_clock_now(g_ceph_context);
-  history.dump_ops(now, &jf);
-  jf.flush(ss);
+  history.dump_ops(now, f);
 }
 
-void OpTracker::dump_ops_in_flight(ostream &ss)
+void OpTracker::dump_ops_in_flight(Formatter *f)
 {
-  JSONFormatter jf(true);
   Mutex::Locker locker(ops_in_flight_lock);
-  jf.open_object_section("ops_in_flight"); // overall dump
-  jf.dump_int("num_ops", ops_in_flight.size());
-  jf.open_array_section("ops"); // list of OpRequests
+  f->open_object_section("ops_in_flight"); // overall dump
+  f->dump_int("num_ops", ops_in_flight.size());
+  f->open_array_section("ops"); // list of OpRequests
   utime_t now = ceph_clock_now(g_ceph_context);
   for (xlist<OpRequest*>::iterator p = ops_in_flight.begin(); !p.end(); ++p) {
-    jf.open_object_section("op");
-    (*p)->dump(now, &jf);
-    jf.close_section(); // this OpRequest
+    f->open_object_section("op");
+    (*p)->dump(now, f);
+    f->close_section(); // this OpRequest
   }
-  jf.close_section(); // list of OpRequests
-  jf.close_section(); // overall dump
-  jf.flush(ss);
+  f->close_section(); // list of OpRequests
+  f->close_section(); // overall dump
 }
 
 void OpTracker::register_inflight_op(xlist<OpRequest*>::item *i)
@@ -109,7 +129,7 @@ void OpTracker::unregister_inflight_op(OpRequest *i)
   utime_t now = ceph_clock_now(g_ceph_context);
   i->xitem.remove_myself();
   i->request->clear_data();
-  history.insert(now, i);
+  history.insert(now, OpRequestRef(i));
 }
 
 bool OpTracker::check_ops_in_flight(std::vector<string> &warning_vector)
@@ -175,12 +195,42 @@ bool OpTracker::check_ops_in_flight(std::vector<string> &warning_vector)
   return warning_vector.size();
 }
 
+void OpTracker::get_age_ms_histogram(pow2_hist_t *h)
+{
+  Mutex::Locker locker(ops_in_flight_lock);
+
+  h->clear();
+
+  utime_t now = ceph_clock_now(NULL);
+  unsigned bin = 30;
+  uint32_t lb = 1 << (bin-1);  // lower bound for this bin
+  int count = 0;
+  for (xlist<OpRequest*>::iterator i = ops_in_flight.begin(); !i.end(); ++i) {
+    utime_t age = now - (*i)->received_time;
+    uint32_t ms = (long)(age * 1000.0);
+    if (ms >= lb) {
+      count++;
+      continue;
+    }
+    if (count)
+      h->set(bin, count);
+    while (lb > ms) {
+      bin--;
+      lb >>= 1;
+    }
+    count = 1;
+  }
+  if (count)
+    h->set(bin, count);
+}
+
 void OpRequest::dump(utime_t now, Formatter *f) const
 {
   Message *m = request;
   stringstream name;
   m->print(name);
   f->dump_string("description", name.str().c_str()); // this OpRequest
+  f->dump_unsigned("rmw_flags", rmw_flags);
   f->dump_stream("received_at") << received_time;
   f->dump_float("age", now - received_time);
   f->dump_float("duration", get_duration());
